@@ -518,6 +518,20 @@ gkr_pk_privkey_serialize (GkrPkObject *obj, const gchar *password, gsize *n_data
 	return gkr_pkix_serialize_private_key_pkcs8 (key->priv->s_key, password, n_data);
 }
 
+static void
+gkr_pk_privkey_lock (GkrPkObject *obj)
+{
+	GkrPkPrivkey *key = GKR_PK_PRIVKEY (obj);
+
+	if (!key->priv->s_key)
+		return;
+	
+	gcry_sexp_release (key->priv->s_key);
+	key->priv->s_key = NULL;
+	
+	initialize_from_key (key);
+}
+
 static void
 gkr_pk_privkey_dispose (GObject *obj)
 {
@@ -536,13 +550,13 @@ gkr_pk_privkey_finalize (GObject *obj)
 {
 	GkrPkPrivkey *key = GKR_PK_PRIVKEY (obj);
 
+	g_assert (!key->priv->pubkey);
+
 	gcry_sexp_release (key->priv->s_key);
 	key->priv->s_key = NULL;
-	
-	initialize_from_key (key);
-	
-	g_assert (!key->priv->pubkey);
-	g_assert (!key->priv->numbers);
+
+	gcry_sexp_release (key->priv->numbers);
+	key->priv->numbers = NULL;	
 	
 	G_OBJECT_CLASS (gkr_pk_privkey_parent_class)->finalize (obj);
 }
@@ -558,6 +572,7 @@ gkr_pk_privkey_class_init (GkrPkPrivkeyClass *klass)
 	parent_class = GKR_PK_OBJECT_CLASS (klass);
 	parent_class->get_attribute = gkr_pk_privkey_get_attribute;
 	parent_class->serialize = gkr_pk_privkey_serialize;
+	parent_class->lock = gkr_pk_privkey_lock;
 	
 	gobject_class = (GObjectClass*)klass;
 	gobject_class->get_property = gkr_pk_privkey_get_property;
@@ -577,20 +592,20 @@ gkr_pk_privkey_new (GkrPkObjectManager *mgr, GQuark location, gcry_sexp_t s_key)
 {
 	GkrPkObject *key;
 	guchar hash[20];
-	gkrid unique; 
+	gkrid digest; 
 	
 	g_return_val_if_fail (s_key != NULL, NULL);
 	
 	if (!gcry_pk_get_keygrip (s_key, hash))
 		g_return_val_if_reached (NULL);
 	
-	/* We need to create a unique for this key */
-	unique = gkr_id_new_digestv ((const guchar*)"private-key", 11, hash, 20, NULL);
+	/* We need to create a digest for this key */
+	digest = gkr_id_new_digestv ((const guchar*)"private-key", 11, hash, 20, NULL);
 	
 	key = g_object_new (GKR_TYPE_PK_PRIVKEY, "manager", mgr, "location", location, 
-	                    "gcrypt-sexp", s_key, "unique", unique, NULL);
+	                    "gcrypt-sexp", s_key, "digest", digest, NULL);
 	                    
-	gkr_id_free (unique);
+	gkr_id_free (digest);
 	
 	return key;
 }
@@ -642,6 +657,9 @@ gkr_pk_privkey_get_keyid (GkrPkPrivkey *key)
 	
 	/* Access via public key */
 	pub = get_public_key (key, TRUE);
+	if(!pub)
+		return NULL;
+	
 	return gkr_pk_pubkey_get_keyid (pub);
 }
 
